"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.closestPoint = exports.validateSeriesPointError = exports.calculateTicksNumber = exports.functionalStyle = exports.validateAxis = exports.validatePosition = exports.splitChildren = exports.getNextId = void 0;
const react_1 = require("react");
const Annotation_1 = require("./components/Annotations/Annotation");
const Axis_1 = __importDefault(require("./components/Axis"));
const ParallelAxis_1 = __importDefault(require("./components/Axis/ParallelAxis"));
const BarSeries_1 = __importDefault(require("./components/BarSeries"));
const Heading_1 = __importDefault(require("./components/Heading"));
const Legend_1 = __importDefault(require("./components/Legend"));
const LineSeries_1 = __importDefault(require("./components/LineSeries"));
const RangeSeries_1 = __importDefault(require("./components/RangeSeries"));
const ScatterSeries_1 = __importDefault(require("./components/ScatterSeries"));
let currentValue = 1;
/**
 * Creates autoincremental values, generally for series.
 */
function getNextId() {
    return ++currentValue;
}
exports.getNextId = getNextId;
/**
 * Validates that all the children inside Plot are supported and organizes them by kind.
 */
function splitChildren(children) {
    const axes = [];
    let heading = null;
    let legend = null;
    let seriesAndAnnotations = [];
    for (let child of react_1.Children.toArray(children)) {
        if (typeof child !== 'object' || !(0, react_1.isValidElement)(child)) {
            // eslint-disable-next-line no-console
            console.error('Invalid Plot child:', child);
            throw new Error('invalid Plot child');
        }
        else if (child.type === LineSeries_1.default ||
            child.type === ScatterSeries_1.default ||
            child.type === RangeSeries_1.default ||
            child.type === BarSeries_1.default ||
            child.type === Annotation_1.Annotations) {
            seriesAndAnnotations.push(child);
        }
        else if (child.type === Axis_1.default || child.type === ParallelAxis_1.default) {
            axes.push(child);
        }
        else if (child.type === Heading_1.default) {
            if (heading !== null) {
                throw new Error('Plot can only have one Heading element');
            }
            heading = child;
        }
        else if (child.type === Legend_1.default) {
            if (legend !== null) {
                throw new Error('Plot can only have one Legend element');
            }
            legend = child;
        }
        else {
            // eslint-disable-next-line no-console
            console.error('Invalid Plot child: ', child);
            throw new Error('invalid plot child');
        }
    }
    return {
        seriesAndAnnotations,
        axes,
        heading,
        legend,
    };
}
exports.splitChildren = splitChildren;
const horizontal = ['top', 'bottom'];
const vertical = ['left', 'right'];
/**
 * Validates that two positions are not orthogonal between them
 */
function validatePosition(currPosition, position, id) {
    const error = (horizontal.includes(currPosition) && !horizontal.includes(position)) ||
        (vertical.includes(currPosition) && !vertical.includes(position));
    if (error) {
        throw new Error(`The positions are ortogonal for ${id}`);
    }
}
exports.validatePosition = validatePosition;
/**
 * Validates that two axes are orthogonal between them
 */
function validateAxis(axisContext, xKey, yKey) {
    const xAxis = axisContext[xKey];
    const yAxis = axisContext[yKey];
    if (!xAxis || !yAxis)
        return [undefined, undefined];
    if (horizontal.includes(xAxis.position)
        ? !vertical.includes(yAxis.position)
        : vertical.includes(xAxis.position)) {
        if (vertical.includes(xAxis.position) ||
            horizontal.includes(yAxis.position)) {
            throw new Error(`The axis ${xKey} should be ${horizontal.join(' ')} and ${yKey} should be ${vertical.join(' ')}`);
        }
        throw new Error(`The axis ${xKey} and ${yKey} are not orthogonal`);
    }
    return [xAxis.scale, yAxis.scale];
}
exports.validateAxis = validateAxis;
/**
 * Checks the style added to a component and if is a function, gets the resulting value
 */
function functionalStyle(defaultStyle, elementStyle, point, index, data) {
    let style = { ...defaultStyle };
    for (const key in elementStyle) {
        if (typeof elementStyle[key] === 'function') {
            style[key] = elementStyle[key](point, index, data);
        }
        else {
            style[key] = elementStyle[key];
        }
    }
    return style;
}
exports.functionalStyle = functionalStyle;
/**
 * Calculate Ticks number to display
 */
function calculateTicksNumber(plotWidth, scientific, domain = [0, 1]) {
    const fontSizeDefault = 16;
    const scientificTickLength = 7;
    let tickLength = `${Math.trunc(domain[1])}`.length;
    // if domain too small => tickLength+2 for decimal values
    tickLength =
        domain[1] - domain[0] < plotWidth * 0.05 ? tickLength + 2 : tickLength;
    const ticksNumber = scientific
        ? plotWidth / (scientificTickLength * fontSizeDefault)
        : plotWidth / (tickLength * fontSizeDefault);
    return ticksNumber;
}
exports.calculateTicksNumber = calculateTicksNumber;
/**
 * validate series point Error
 */
function validateSeriesPointError(error) {
    if (typeof error === 'number')
        return [error, error];
    else if (Array.isArray(error) && error.length >= 2)
        return error;
    return null;
}
exports.validateSeriesPointError = validateSeriesPointError;
function closestPoint(data, value, distanceFun) {
    let closest = {
        index: 0,
        distance: Infinity,
    };
    for (let i = 0; i < data.length; i++) {
        const distance = distanceFun(data[i], value);
        if (distance < closest.distance) {
            closest.index = i;
            closest.distance = distance;
        }
    }
    return data[closest.index];
}
exports.closestPoint = closestPoint;
